# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
class Hermes::Api::Dmo::ApplicantsController < Hermes::Api::Dmo::ApplicationController
  hermes_tools 'aggregations'
  require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s
  set_instance_for :eligible_requests

  MANAGEMENT_ROLES = %w[
    Administrator
    Manager
  ].freeze

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ].freeze

  ACCEPTABLE_PARAMS = %w[
    ids
    nud
    temporary_id
    sne_profile_creation_date
    ss_id
    first_name
    last_name
    patronymic_name
    nationality
    email
    email_advances_assistant
    event_notification
    civility
    birth_date
    work_contract_type
    profession_details
    has_other_employers
    main_employer_name
    main_employer_siret
    workplace_postal_code
    workplace_locality
    foreign_workplace_postal_code
    foreign_workplace_locality
    foreign_workplace_country
    year_minus_one
    amount_year_minus_one
    year_minus_two
    amount_year_minus_two
    ressources_details
    tax_revenues
    apl_amount
    postal_details
    actual_housing_details
    family_situation
    family_situation_validated
    asked_children_nb
    valid_children_nb
    asked_family_size
    valid_family_size
    situation_tax_treshold
    status
    status_updated_at
    asked_locataire_hlm
    valid_locataire_hlm
    valid_priority_public
    asked_priority_public
    single_parent
    priority_public
    mobility
    mutation
    completed_requests_nb
    on_going_request_nb
    disability
    renewal_date
    eligibility_query
    active_housing_request_id
    housing_request_ids
    entity_id
    application_ids
    monthly_incomes
    monthly_charges
    cellphone
    sms_notification
    homephone
    workphone
    main_user_id
    address
    postal_code
    district
    country
    latitude
    longitude
  ].freeze

  REGEX_PARAMS = %w[
  ].freeze

  MUTATE_PARAMS = %i[
    workplace_insee_code
    temporary_id
    nud
    sne_profile_creation_date
    ss_id
    first_name
    last_name
    patronymic_name
    nationality
    email
    email_advances_assistant
    event_notification
    civility
    birth_date
    work_contract_type
    profession_details
    has_other_employers
    main_employer_name
    main_employer_siret
    workplace_postal_code
    workplace_locality
    foreign_workplace_postal_code
    foreign_workplace_locality
    foreign_workplace_country
    year_minus_one
    amount_year_minus_one
    year_minus_two
    amount_year_minus_two
    ressources_details
    tax_revenues
    apl_amount
    postal_details
    actual_housing_details
    family_situation
    family_situation_validated
    situation_tax_treshold
    status
    asked_locataire_hlm
    valid_locataire_hlm
    valid_priority_public
    asked_priority_public
    priority_public
    mobility
    mutation
    disability
    renewal_date
    monthly_incomes
    monthly_charges
    cellphone
    is_owner
    sms_notification
    homephone
    workphone
    main_user_id
    address
    postal_code
    district
    country
    latitude
    longitude
  ].freeze
  mutate_params_only(*MUTATE_PARAMS)

  def index
    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS
    super
  end

  #   def create
  #     user_id = @user.id
  #     param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
  #     Rails.logger.debug "TEST before #{user_id}"
  #     user_id = param_user if @user.has_role?('Administrator') && param_user
  #     Rails.logger.debug "TEST #{mutate_valid_params} after #{user_id}"
  #     @instance = @model.new(mutate_valid_params.merge(writers: [user_id.to_s], procurating_user_id: user_id.to_s))
  #
  #     return respond_with_errors(@instance) unless @instance.save
  #
  #     render(jsonapi: @instance, serializer: @serializer, status: :created)
  #   end

  OFFERS_ACCEPTABLE_PARAMS = %w[
    created_at
    updated_at
    eligibility_query
    typology
    lessor_status
    type
    age
    district
    department
    publication_end_date
    rent_amount
    surface
    rooms
    exclusivity_end_date
    nb_of_eligible_employees
    nb_of_requests_from_employees
    availability_date
    description
    details
    old_status
    status
    status_updated_at
    effort_rate
    min_effort_treshold
    single_parent
    reserved
    priority_public
    mobility
    bedrooms
    bathrooms
    heating_type
    floors
    has_elevator
    has_gardian
    dpe
    outside
    has_cellar
    parking_type
    pmr
    kitchen_type
    mutation
    eligibility_string
    threshold_category_1
    threshold_type
    threshold_subtype
    threshold_category_2
    threshold_category_3
    threshold_category_4
    threshold_category_5
    threshold_category_6
    threshold_category_7
    threshold_category_8
    threshold_category_9
    threshold_category_10
    threshold_category_11
    threshold_category_12
    individual_heating
    date_publication_start
    date_publication_end
    lessor_id
    kind
    building_type
    allready_rent
    guarantee_deposit
    rental_charges
    car_park_rent
    offer_status
    offer_status_updated_at
    lessor_status_updated_at
    ceil_category_1
    ceil_category_2
    ceil_category_3
    ceil_category_4
    ceil_category_5
    ceil_category_6
    ceil_category_7
    ceil_category_8
    ceil_category_9
    ceil_category_10
    ceil_category_11
    ceil_category_12
    floor_category_1
    floor_category_2
    floor_category_3
    floor_category_4
    floor_category_5
    floor_category_6
    floor_category_7
    floor_category_8
    floor_category_9
    floor_category_10
    floor_category_11
    floor_category_12
    min_revenues
    total_charges
    max_effort_rate
    charges
    tense_area
    subsidiary_company
    rpls_id
    operation_name
    administrator
    administration_area
    potential_request_name_town
    potential_request_name_county
    number_of_companies_30_minutes_around
    construction_year
    address_can_be_shared
    qpv
    total_floors
    address_extended
    dpe_gas
    test_created_at
    new_offer
    lessor_name
    lessor_siren
    als_siren
    postal_code
    residence_code
    pictures
    agency_id
    agency_name
    lessor_contact_name
    lessor_email
    lessor_phone_number
    garden
    green_area
    condo
    furnished_lodging
    virtual_visit
    duplex
    first_quartile
    other_quartiles
    lessor_convention_reference
    reserving_agency_name
    reserving_agency_siren
    is_owner
    address
    postal_code
    country
    latitude
    longitude
  ].freeze

  OFFERS_REGEX_PARAMS = %w[
  ].freeze

  def set_instance
    instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
    if (instance && (instance.readable_by?(@user) || instance&.entity&.readable_by?(@user)))
      @instance = instance
    else
      nil
    end
  end

  def user_is_allowed_to_remove_entity
    @instance.writable_by?(@user) || @instance.entity&.writable_by?(@user)
  end

  def allow_entity_id_remove (mutate_valid_params)
    entity_id_has_been_removed = params[:data][:attributes].key?("entity_id") && params[:data][:attributes][:entity_id]=== nil
    siret_has_been_removed = params[:data][:attributes].key?("main_employer_siret") && params[:data][:attributes][:main_employer_siret]=== nil
    if (entity_id_has_been_removed || siret_has_been_removed) && user_is_allowed_to_remove_entity
      mutate_valid_params[:entity] = nil
      mutate_valid_params[:main_employer_siret] = nil
    end
    mutate_valid_params
  end

  def all_eligibile_offers
    eligibility = JSON.parse(@instance.eligibility_query).each_with_object({}) do |(key, val), h|
      h[key.to_sym] = val
    end
    filter = build_query(Hermes::HousingOffer).deep_merge(eligibility)

    Hermes::HousingOffer.where(filter).order(order_params)
  end

  def update
    return render_json_error(:not_found, :object_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && (@instance.writable_by?(@user) || @instance&.entity.writable_by?(@user))

    valid_params = allow_entity_id_remove(mutate_valid_params)

    param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
    valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
    return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
    Rails.logger.debug "TEST valid_params are #{valid_params}"
    return respond_with_errors(@instance) unless @instance.update(valid_params) && @instance.save

    render(jsonapi: @instance, serializer: @serializer)
  end

  def eligible_offers
    return render_json_error(:not_found, :instance_not_found) unless @instance
    unless can_manage_requests?
      return render_json_error(:unprocessable_action, :not_manager)
    end

    acceptable_params *OFFERS_ACCEPTABLE_PARAMS
    regex_params *OFFERS_REGEX_PARAMS
    valid_operators *VALID_OPERATORS
    options = {}
    aggregation = all_eligibile_offers
    scoring_params = params.slice(*SCORING_PARAMS)
    scoring_sorting_params = params[:sort]&.slice(*SCORING_PARAMS)
    if (scoring_params.present? || scoring_sorting_params.present?)
      aggregation = aggregate_with_score('offers',  @instance.id, params, aggregation)
      options[:entries_class] = Hermes::HousingOffer
      options[:add_scores] = true
    else
      limited_aggregation = aggregation.only(:id).offset((page - 1) * per_page).limit(per_page)
      scores = Hermes::Score.where(:housing_offer_id.in => limited_aggregation.pluck(:id), applicant: @instance)
      options.merge!(scores: scores)
    end

    paginate(
      aggregation,
      each_serializer: Hermes::HousingOfferSerializer,
      **options
    )
  end

  def find_by_nud
    @instance = @model.find_or_create_by_nud(params[:nud], params[:birth_date])
    if @instance.errors.present?
      return render_json_error(:unprocessable_entity, :invalid_birth_date) if @instance.errors.messages[:birth_date]
      return render_json_error(:not_found, :invalid_nud) if @instance.errors.messages[:nud]
    end

    render(jsonapi: @instance, serializer: @serializer)
  end

  def switch
    return render_json_error(:not_found, :instance_not_found) unless @instance

    @target = User.find_by(main_applicant_id: @instance.id)
    @target = User.find_by(email: @instance.email) unless @target
    return render(json: { success: false, message: 'Utilisateur target non trouvé.' }, status: :not_found) unless @target
    return render nothing: true, status: :not_found unless @user

    if current_user_roles.include?('entreprise')
      delegated_switch_response
    else
      render json: {
        success: false,
        message: 'Votre role doit être \'entreprise\' pour effectuer cette action.'
      }, status: :unauthorized
    end
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
  end

  private

  def user_can_destroy?
    (@instance.user_id.to_s == current_user.id.to_s) ||
      current_user.has_role?('Administrator') ||
      current_user.has_role?('Manager') ||
      current_user.has_role?('Superadmin')
  end

  def can_manage_requests?
    (MANAGEMENT_ROLES & current_user_roles)&.length&.send('>', 0)
  end

  def delegated_switch_response
    params.require(:entity_id)
    entity = Hermes::Entity.readable_by(@user).find_by(id: params[:entity_id])
    return render(json: { success: false, message: 'Entity not found' }, status: :not_found) unless entity

    entity_form = Hermes::HousingRequestEntityForm.find_by(entity_id: entity.id, applicant_id: @instance.id)
    return render(json: { success: false, message: 'delegation not found' }, status: :not_found) unless entity_form

    if entity_form.present? && entity_form.delegation && entity_form.delegation_signed
      success_switch_response
    else
      render json: {
        success: false,
        message: 'delegation not available or not signed'
      }, status: :unauthorized
    end
  end

  def success_switch_response
    token = @target.generate_jwt(true_user_id: @user.id.to_s)
    render json: {
      success: true,
      info: t(:registered),
      email: @target.email,
      login: @target.login,
      jwt_token: token
    }, status: :ok
  end
end
