# frozen_string_literal: true

class UpdateRankJob < ActiveJob::Base
 
  queue_as :x_hermes_rank
  unique :until_executed

  def perform(offer_id)
    if offer = Hermes::HousingOffer.find_by(id: offer_id)
      eligible_position = 1
      applicated_position = 1
      
      scores = Hermes::Score.only(
        :id, 
        :eligible_position, 
        :applicated_position, 
        :profile_epci_score,
        :sne_date, 
        :updated_at,
        :housing_application_id,
        :has_applicated
      ).where(
        housing_offer_id: offer_id, 
        is_active: true
      ).order_by(
        profile_epci_score: :desc,
        sne_date: :asc,
        has_applicated: :desc,
      )

      application_to_update = []

      bulk_order = []
      scores.each do |score|
        if score.eligible_position != eligible_position || score.applicated_position != applicated_position
          bulk_order.push(
            {
              update_one:
              {
                filter: { _id: score.id },
                update: { :$set => {
                  eligible_position: eligible_position,
                  applicated_position: applicated_position,
                  updated_at: DateTime.current
                }}
              }
            }
          )
        end
        eligible_position += 1
        applicated_position += 1 if score.has_applicated
        application_to_update += [score.housing_application_id] if score.housing_application_id && score.has_applicated
      end
      Hermes::Score.collection.bulk_write(bulk_order)

      application_to_update.each do |id|
        if application = Hermes::HousingApplication.find(id)
          application.update_application_score
          application.save if application.changed?
        end 
      end

      Rails.logger.debug "Updated ranking for housing_offer : #{offer_id}"
    end
  end
end