# frozen_string_literal: true

module Hermes
  class HousingChannelSetting
    include Hermes::Concerns::CommonFieldsConcern

    hermes_errors 'housing_channel_error'

    writer_roles 'Administrator'
    reader_roles 'Administrator'
    owner_field :user_id

    field :department, type: String
    field :insee_code, type: String
    field :housing_channel_keys, type: Array, default: []

    validates_presence_of :department

    common_indexes *%i[
      department
      insee_code
      housing_channel_keys
    ]

    before_save :before_save_actions

    def check_housing_channel_keys
      valid_channel_keys = Hermes::HousingChannel.where(:key.in => housing_channel_keys).pluck(:key).uniq
      invalid_keys = housing_channel_keys - valid_channel_keys

      raise Hermes::Errors::HousingChannelError, I18n.t('errors.housing_channel.not_found', key: invalid_keys.join(', ')) if invalid_keys.any?
    end

    private

    def before_save_actions
      check_housing_channel_keys if housing_channel_keys_changed?
    end
  end
end