# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::Mappings::ApplicantReverseMap, type: :helper do
  let!(:sne_request_with_valid_years) do
    {"interfaceNuu"=>{"xmlns:ns1"=>"http://nuu.application.i2/", "versionInterface"=>"03.00", "entete"=>{"typeFichier"=>"DIS", "dateFichier"=>"2020-07-30T11:51:46"}, "demande"=>{"demandeLogement"=>{"numGuichet"=>"G1175", "numUnique"=>"0130620003912G1U05", "refInterne"=>"000002048", "dateCreationDemande"=>"2020-06-01", "forcerDoublon"=>"false", "renouvellementElec"=>"false", "precision"=>nil, "ressourcesLoiEC"=>"0", "informationTraitement"=>{"estDelaiAnormalementLong"=>"false", "estMultiplesAttributionsSousConditions"=>"false", "dateCalculIndicateur"=>"2020-07-26T00:00:00"}, "logementRecherche"=>{"categorieLogementRecherche"=>{"code"=>"I"}, "rezDeChaussee"=>"true", "sansAscenseur"=>"true", "montantLoyer"=>"0", "demandeElargie"=>"true", "listeTypeLogement"=>{"typeLogement"=>{"code"=>"T2"}}, "listeLocalisationSouhaite"=>{"localisationSouhaite"=>{"commune"=>{"code"=>"13001"}, "codePostal"=>"13085", "seuilEPCIPremierQuartile"=>"7718", "quartileAppartenance"=>"NR"}}}, "principauxMotifDemande"=>{"motifDemande"=>{"code"=>"10"}}, "personnePhysique"=>{"demandeur"=>{"civilite"=>{"code"=>"1"}, "nom"=>"PLATO", "nomPatronymique"=>nil, "prenom"=>"Tom", "dateNaissance"=>"1989-01-01", "nationalite"=>{"code"=>"1"}, "situationFamiliale"=>{"code"=>"C"}, "mel"=>"guyactionlogement+test@gmail.com", "notifSms"=>"false", "notifEvt"=>"false", "adressePostale"=>{"etranger"=>"false", "voie"=>"bd de la liberté", "codePostal"=>"69003", "commune"=>{"code"=>"69383"}}, "situationProfessionnelle"=>{"typeContratTravail"=>{"code"=>"CHO"}, "multiEmployeur"=>"false"}, "revenuFiscal"=>{"anneeMoins1"=>(DateTime.current.year - 1).to_s, "montantMoins1"=>"5000", "anneeMoins2"=>(DateTime.current.year - 2).to_s, "montantMoins2"=>"1000"}, "listeRessourceRecue"=>nil, "listeRessourceVersee"=>nil, "listeRessourceRecuePersACharge"=>nil, "listeRessourceVerseePersACharge"=>nil}, "adresseLogement"=>{"etranger"=>"false", "voie"=>"bd de la liberté", "codePostal"=>"69003", "commune"=>{"code"=>"69383"}}, "listeCotitulaire"=>{"cotitulaire"=>{"civilite"=>{"code"=>"1"}, "nom"=>"PLATO", "nomPatronymique"=>nil, "prenom"=>"Tim", "dateNaissance"=>"1989-01-01", "nationalite"=>{"code"=>"1"}, "situationFamiliale"=>{"code"=>"C"}, "lienDemandeur"=>{"code"=>"L"}, "situationProfessionnelle"=>{"typeContratTravail"=>{"code"=>"CHO"}, "multiEmployeur"=>"false"}, "revenuFiscal"=>nil, "listeRessourceRecue"=>nil, "listeRessourceVersee"=>nil}}, "logementActuel"=>{"listeModeleLogement"=>{"modeleLogement"=>{"code"=>"LP"}}, "montantMensuelLoyer"=>"0", "montantMensuelAPL"=>"0", "nombreHabitant"=>"0", "surface"=>"0", "proprietaire"=>"false", "etranger"=>"false"}}}}}}
  end
  let!(:sne_request_with_invalid_years_but_inheritable) do
    {"interfaceNuu"=>{"xmlns:ns1"=>"http://nuu.application.i2/", "versionInterface"=>"03.00", "entete"=>{"typeFichier"=>"DIS", "dateFichier"=>"2020-07-30T11:51:46"}, "demande"=>{"demandeLogement"=>{"numGuichet"=>"G1175", "numUnique"=>"0130620003912G1U05", "refInterne"=>"000002048", "dateCreationDemande"=>"2020-06-01", "forcerDoublon"=>"false", "renouvellementElec"=>"false", "precision"=>nil, "ressourcesLoiEC"=>"0", "informationTraitement"=>{"estDelaiAnormalementLong"=>"false", "estMultiplesAttributionsSousConditions"=>"false", "dateCalculIndicateur"=>"2020-07-26T00:00:00"}, "logementRecherche"=>{"categorieLogementRecherche"=>{"code"=>"I"}, "rezDeChaussee"=>"true", "sansAscenseur"=>"true", "montantLoyer"=>"0", "demandeElargie"=>"true", "listeTypeLogement"=>{"typeLogement"=>{"code"=>"T2"}}, "listeLocalisationSouhaite"=>{"localisationSouhaite"=>{"commune"=>{"code"=>"13001"}, "codePostal"=>"13085", "seuilEPCIPremierQuartile"=>"7718", "quartileAppartenance"=>"NR"}}}, "principauxMotifDemande"=>{"motifDemande"=>{"code"=>"10"}}, "personnePhysique"=>{"demandeur"=>{"civilite"=>{"code"=>"1"}, "nom"=>"PLATO", "nomPatronymique"=>nil, "prenom"=>"Tom", "dateNaissance"=>"1989-01-01", "nationalite"=>{"code"=>"1"}, "situationFamiliale"=>{"code"=>"C"}, "mel"=>"guyactionlogement+test@gmail.com", "notifSms"=>"false", "notifEvt"=>"false", "adressePostale"=>{"etranger"=>"false", "voie"=>"bd de la liberté", "codePostal"=>"69003", "commune"=>{"code"=>"69383"}}, "situationProfessionnelle"=>{"typeContratTravail"=>{"code"=>"CHO"}, "multiEmployeur"=>"false"}, "revenuFiscal"=>{"anneeMoins1"=>(DateTime.current.year - 2).to_s, "montantMoins1"=>"5000", "anneeMoins2"=>(DateTime.current.year - 3).to_s, "montantMoins2"=>"1000"}, "listeRessourceRecue"=>nil, "listeRessourceVersee"=>nil, "listeRessourceRecuePersACharge"=>nil, "listeRessourceVerseePersACharge"=>nil}, "adresseLogement"=>{"etranger"=>"false", "voie"=>"bd de la liberté", "codePostal"=>"69003", "commune"=>{"code"=>"69383"}}, "listeCotitulaire"=>{"cotitulaire"=>{"civilite"=>{"code"=>"1"}, "nom"=>"PLATO", "nomPatronymique"=>nil, "prenom"=>"Tim", "dateNaissance"=>"1989-01-01", "nationalite"=>{"code"=>"1"}, "situationFamiliale"=>{"code"=>"C"}, "lienDemandeur"=>{"code"=>"L"}, "situationProfessionnelle"=>{"typeContratTravail"=>{"code"=>"CHO"}, "multiEmployeur"=>"false"}, "revenuFiscal"=>nil, "listeRessourceRecue"=>nil, "listeRessourceVersee"=>nil}}, "logementActuel"=>{"listeModeleLogement"=>{"modeleLogement"=>{"code"=>"LP"}}, "montantMensuelLoyer"=>"0", "montantMensuelAPL"=>"0", "nombreHabitant"=>"0", "surface"=>"0", "proprietaire"=>"false", "etranger"=>"false"}}}}}}
  end
  let!(:sne_request_with_invalid_years) do
    {"interfaceNuu"=>{"xmlns:ns1"=>"http://nuu.application.i2/", "versionInterface"=>"03.00", "entete"=>{"typeFichier"=>"DIS", "dateFichier"=>"2020-07-30T11:51:46"}, "demande"=>{"demandeLogement"=>{"numGuichet"=>"G1175", "numUnique"=>"0130620003912G1U05", "refInterne"=>"000002048", "dateCreationDemande"=>"2020-06-01", "forcerDoublon"=>"false", "renouvellementElec"=>"false", "precision"=>nil, "ressourcesLoiEC"=>"0", "informationTraitement"=>{"estDelaiAnormalementLong"=>"false", "estMultiplesAttributionsSousConditions"=>"false", "dateCalculIndicateur"=>"2020-07-26T00:00:00"}, "logementRecherche"=>{"categorieLogementRecherche"=>{"code"=>"I"}, "rezDeChaussee"=>"true", "sansAscenseur"=>"true", "montantLoyer"=>"0", "demandeElargie"=>"true", "listeTypeLogement"=>{"typeLogement"=>{"code"=>"T2"}}, "listeLocalisationSouhaite"=>{"localisationSouhaite"=>{"commune"=>{"code"=>"13001"}, "codePostal"=>"13085", "seuilEPCIPremierQuartile"=>"7718", "quartileAppartenance"=>"NR"}}}, "principauxMotifDemande"=>{"motifDemande"=>{"code"=>"10"}}, "personnePhysique"=>{"demandeur"=>{"civilite"=>{"code"=>"1"}, "nom"=>"PLATO", "nomPatronymique"=>nil, "prenom"=>"Tom", "dateNaissance"=>"1989-01-01", "nationalite"=>{"code"=>"1"}, "situationFamiliale"=>{"code"=>"C"}, "mel"=>"guyactionlogement+test@gmail.com", "notifSms"=>"false", "notifEvt"=>"false", "adressePostale"=>{"etranger"=>"false", "voie"=>"bd de la liberté", "codePostal"=>"69003", "commune"=>{"code"=>"69383"}}, "situationProfessionnelle"=>{"typeContratTravail"=>{"code"=>"CHO"}, "multiEmployeur"=>"false"}, "revenuFiscal"=>{"anneeMoins1"=>(DateTime.current.year - 3).to_s, "montantMoins1"=>"5000", "anneeMoins2"=>(DateTime.current.year - 4).to_s, "montantMoins2"=>"1000"}, "listeRessourceRecue"=>nil, "listeRessourceVersee"=>nil, "listeRessourceRecuePersACharge"=>nil, "listeRessourceVerseePersACharge"=>nil}, "adresseLogement"=>{"etranger"=>"false", "voie"=>"bd de la liberté", "codePostal"=>"69003", "commune"=>{"code"=>"69383"}}, "listeCotitulaire"=>{"cotitulaire"=>{"civilite"=>{"code"=>"1"}, "nom"=>"PLATO", "nomPatronymique"=>nil, "prenom"=>"Tim", "dateNaissance"=>"1989-01-01", "nationalite"=>{"code"=>"1"}, "situationFamiliale"=>{"code"=>"C"}, "lienDemandeur"=>{"code"=>"L"}, "situationProfessionnelle"=>{"typeContratTravail"=>{"code"=>"CHO"}, "multiEmployeur"=>"false"}, "revenuFiscal"=>nil, "listeRessourceRecue"=>nil, "listeRessourceVersee"=>nil}}, "logementActuel"=>{"listeModeleLogement"=>{"modeleLogement"=>{"code"=>"LP"}}, "montantMensuelLoyer"=>"0", "montantMensuelAPL"=>"0", "nombreHabitant"=>"0", "surface"=>"0", "proprietaire"=>"false", "etranger"=>"false"}}}}}}
  end

  describe 'Applicant datas populate' do
    it 'should populate amount_minus_one and amount_minus_two' do
      app = Hermes::Applicant.new(Hermes::BoClient::Mappings::ApplicantReverseMap.new(sne_request_with_valid_years, true).build_applicant)
      expect(app.year_minus_one).to eq((DateTime.current.year - 1).to_s)
      expect(app.year_minus_two).to eq((DateTime.current.year - 2).to_s)
      expect(app.amount_year_minus_one).to eq('5000'.to_f)
      expect(app.amount_year_minus_two).to eq('1000'.to_f)
    end

    it 'should not populate amount_minus_one and amount_minus_two with amount_minus_one from sne' do
      app = Hermes::Applicant.new(Hermes::BoClient::Mappings::ApplicantReverseMap.new(sne_request_with_invalid_years_but_inheritable, true).build_applicant)
      expect(app.year_minus_one).to eq((DateTime.current.year - 1).to_s)
      expect(app.year_minus_two).to eq((DateTime.current.year - 2).to_s)
      expect(app.amount_year_minus_one).to be_nil
      expect(app.amount_year_minus_two).to eq('5000'.to_f)
    end

    it 'should not populate amount_minus_one and amount_minus_two with sne values' do
      app = Hermes::Applicant.new(Hermes::BoClient::Mappings::ApplicantReverseMap.new(sne_request_with_invalid_years, true).build_applicant)
      expect(app.year_minus_one).to eq((DateTime.current.year - 1).to_s)
      expect(app.year_minus_two).to eq((DateTime.current.year - 2).to_s)
      expect(app.amount_year_minus_one).to be_nil
      expect(app.amount_year_minus_two).to be_nil
    end
  end
end
